

#ifndef __MODULE_UTILITIES__
#define __MODULE_UTILITIES__

#ifdef __cplusplus
extern "C" {
#endif

#define maxNrItems 100		/* max number of items in a dialog */

typedef enum { unknownItem=0, windowPopupItem, columnPopupItem } ItemType;	// the item types


typedef struct
{
	ItemType	type;			// the type of an item, unknownItem if item needs no special treatement				
	long		n1;				// used to store special information for some items
								//	for dataColumnPopups: 0 if all columns, 1 if only numerical columns, 2 if only text columns
	long		n2;				// used to store special information for some items
								//	for dataColumnPopups: 0 or the item id of a corresponding window popup
	long		n3;				// used to store special information for some items
								//	for dataColumnPopups: 0 or the windowRefID it was last updated for
	void*		data;			// used to store information on some items
								// for windowPopupItem: handle to an array of windowRefIDs
} ItemRecord;

typedef struct
{
	DialogPtr		dialog;					// window pointer to the dialog
	short			okItem, cancelItem;		// the items to be activated on return/cmd-. (0 if none)
	short			itemHit;				// the item to be returned from MyModalDialog
	ItemRecord		items[maxNrItems];		// an array with information on some of the items
	long			timer;					// != 0 if timer is enabled (see MySetTimer)
	Rect			frames[4];				// rects to frame (0,0,0,0) if no frame (See MySetFrame)
	ControlRef		rootControl;			// the window's root control (or nil if embedding not used)
}DialogData;

DialogData* MyGetNewDialog(short resID, short okItem, short cancelItem);
	// call instead of GetNewDialog
	// resID:		the resource ID of the DLOG
	// okItem:		the item to be activated when Enter/Return is pressed (0 if none)
	// cancleItem	the item to be activated when Escape/cmd-. is pressed (0 if none)
	// returns nil if failure

void MyDisposeDialog(DialogData* dlg);
	// call instead of DisposeDialog
	// dlg: the data returned by MyGetNewDialog


void MyModalDialog(DialogData* dlg, short* const itemHit);
	// call instead of ModalDialog
	// dlg: the data returned by MyGetNewDialog
	// Automatically toggles checkboxes when clicked

void MyShowDialog(DialogData* dlg);
	// makes the dialog visible


// routines for accessing items:

ControlRef MyGetItemAsControl(DialogData* dlg, short item);
	// returns the control handle of an item, works only if the dialog uses embedding

MenuHandle MyGetItemMenuHandle(DialogData* dlg, short item);
	// returns the menu handle if a menu is associated with this item
	// Works only if the dialog uses the appearance manager

void MyGetItemText(DialogData* dlg, short item, Str255 dest);
	// returns the text in an item

void MySetItemText(DialogData* dlg, short item, const Str255 s);
	// sets the text of an item

void MySelectItemText(DialogData* dlg, short id, short start, short end);
	// selects a text edit item

void MySetExtendedItem(DialogData* dlg, short id, double val);
	// sets a text item to a floating point number

Boolean MyGetExtendedItem(DialogData* dlg, short id, double* const val);
	// retrieves a floating point number from a text item
	// if conversion fails: val is unchanged, the text is selected, and false is returned

void MySetLongItem(DialogData* dlg, short id, long val);
	// sets a text item to a longint number

Boolean MyGetLongItem(DialogData* dlg, short id, long* const val);
	// retrieves a longint number from a text item
	// if conversion fails: val is unchanged, the text is selected, and false is returned


ControlHandle MyGetItemHandle(DialogData* dlg, short item);
	// returns the control handle of an item

void MyHiliteItem(DialogData* dlg, short item, short value);
	// hilites an item (value == 255 for disabling)

void MyShowItem(DialogData* dlg, short item);
	// makes the given item visible

void MyHideItem(DialogData* dlg, short item);
	// makes the given item invisible

Rect MyGetItemRect(DialogData* dlg, short item);
	// returns enclosing rect of an item

void MyInvalidateItem(DialogData* dlg, short item);
	// invalidates the given item

void MySetItemValue(DialogData* dlg, short item, short value);
	// sets the value of a control an item

short MyGetItemValue(DialogData* dlg, short item);
	// gets the value of an item
	
void SetItemGeneral(DialogData* dlg, short item, void* data);
	// generic routine for setting items. Presently recognized by
	//	window popup items:	data must be a pointer to a long value, containing the window ref ID

void GetItemGeneral(DialogData* dlg, short item, void* data);
	// generic routine getting the setting of items. Presently recognized by
	//	window popup items:	data must be a pointer to a long value, where the window ref ID is returned


void FlashControl(ControlHandle control);
	// hilites the control for a moment

// special popup items:

void MakeWindowPopup(DialogData* dlg, short item, long windowType);
	// adds a list of windows to the given popup item
	// windowType can be drawingType, dataType or textType and defines
	// the window type that should be added to the menu
	// use GetItemGeneral to get the selected window

void MakeColumnPopup(DialogData* dlg, short item, short windowPopup, short what);
	// adds a list of column names to a popup item
	//	windowPopup:	the item id of a window popup that this item belongs to, 0 if none
	//	what:			0 if all columns, 1 if only data columns, 2 if only text columns
	// use GetItemValue/SetItemValue to access the column
	// if the column popup belongs to a window popup, first call MakeWindowPopup for the window popup

long GetSelectedColumn(DialogData* dlg, short item);
	// returns the index of the column currently selected in the popup.

void SetSelectedColumn(DialogData* dlg, short item, long columnIndex);
	// sets the selected column in the popup.
	

// setting the timer of a dialog:

void MySetTimer(DialogData* dlg, long ticks);
	// Each dialog can have a timer, which is disabled by default. To enable the timer,
	// call the routine MySetTimer with a value if ticks that is != 0
	// If the timer is enabled, it MyModalDialog check if TickCount() > ticks. If yes,
	// ticks is reset to 0 and MyModalDialog returns with itemHit = -1.

// setting frames

void MySetFrame(DialogData* dlg, Rect* const r, short index);
	// Sets a frame that must be drawn whenever the dialog box is updated
	// Set r to nil if you want to clear a frame
	// There are up to 4 frames. Each is identified by its index (0..3)

// appending and removing DITLs

void MyAppendDITL(DialogData* dlg, short resID);
	// appens DITL of given res-id to dialog

void MyShortenDITL(DialogData* dlg, short nrItems);
	// removes a given number of items from dlg

// alert boxes

short MyAlert(short resID);
	// shows a given alert box, returns the clicked item
	// resID: the res ID of a DLOG

short MyPositionedAlert(short resID, Rect* const position);
	// same as MyAlert, but allows setting and storing of window position
	// position: on entry:	the position of the alert window in global coord. or {0,0,0,0}
	//						for default position
	//			 on exit:	the last position of the alert window


// miscellaneous utilities:

void AddItemToMenu(MenuHandle menu, Str255 name);
	// adds an item with the given name to the given menu
	// makes sure that the name contains no illegal characters that would mess up the menu

double RoundDouble(double x);
	// rounds x to next integer


// window handling:

// this utility package is able to handle several windows at the same time.
// This is only possible if it has a list of its windows and if it knows
// how to handle update and actvate/deactivate events for each window.
// For this purpose, each window that the module creates must be registered 
// by calling RegisterWindow/UnregisterWindow

typedef Boolean (*WindowHandler)(long message, void* m, void* param);
	// a routine for handling the window.
	// message tells you what the routine should do. m depends on message.
	// param is the parameter passed to RegisterWindow.
	// The routine must return false if it does not recognize or handle a given message.
	// presently used messages:
	//	'evnt':	Handle a window related event. m points to the event record.
	//			Return true if the event loop should exit, false otherwise.
	//  'grw1':	Called when the grow button was called. m points to a Rect.
	//			set (*m).top/left to the minimum size of the window, 
	//			(*m).bot/right to its maximum size and return true. If the window
	//			cannot be grown, return false.
	//	'grw2':	Called when the window has been resized.
	//	'menu':	Called after click into menu. m points to the long value returned by MenuSelct
	//			If implemented, your handler should return true if we should
	//			exit the event loop.
	// Your WindowHandler must implement the 'evnt' message, all other messages are
	// optional. If you do not handle a message, return false.


void RegisterWindow(WindowPtr window, WindowHandler handler, void* param);
	// register a private window of this module
	// window: the window ptr to it
	// handler: called for handling window related events, must at least implement update events
	// param: parameter passed each time handler is called

void WindowCanGrow(WindowPtr window, Rect* const growRect);
	// if a window can be grow, call this routine after having called RegisterWindow
	// growRect must contain the minimum size of the window in top/left and the
	// maximum size in bottom/right

void UnregisterWindow(WindowPtr window);
	// unregister a private window previously registered by RegisterWindow
	// If the window handler routine implements the 'rect' method, the rectangle will
	// be updated to the last position of the window
	// UnregisterWindow must be called before the window is deallocated 

void EventLoop(void);
	// Gets events and handles them until a window handler returns true

void GetGlobalWindowRect(WindowPtr window, Rect* const r);
	// returns the window's rect in global coordinates

void SetGlobalWindowRect(WindowPtr window, Rect* const r);
	// moves the window to the given global coordinates

void HandleUpdateOrActivateEvent(EventRecord * const event);
	// handles update or activate event in a pro fit window or in a registered window


typedef void (*PickerCallbackPtr)(RGBColor* const newColor, void* data);

Boolean MyGetColor(Str255 const title, RGBColor* const in, RGBColor* const out,
		const PickerMenuItemInfo* const mInfo, PickerCallbackPtr colorChangeProc, void* callbackData);
	// brings up the color picker
	// mInfo: set to the editMenuID end the cut, copy, etc items therein
	// colorChangeProc: called whenever the user changes the caller. Pass nil if not used
	// callbackData: data passed to colorChangeProc


	
Boolean MyGetFileSpecification(FSSpec * theFile, OSType fileType1, OSType fileType2);
	// Puts up an "Open File" dialog to get a file choice from the user.
	// fileType1 and fileType2 which file types must be displayed in the dialog. Use '****' to display all types
	// Set fileType2 to 0 if you do not need it.s
	

Boolean MyGetFileRef(FSRef* const theFile, OSType fileType1, OSType fileType2);
	// Puts up an "Open File" dialog to get a file choice from the user.
	// fileType tells which file types must be displayed in the dialog. Use '****' to display all types

	
	#if !defined(ACCESSOR_CALLS_ARE_FUNCTIONS) || !ACCESSOR_CALLS_ARE_FUNCTIONS
		OSErr AEGetDescData(const AEDesc * theAEDesc, void * dataPtr, Size maximumSize);
		// extracts data from a descriptor record
	
		Size AEGetDescDataSize(const AEDesc * theAEDesc);
		// returns the size of the data in a descriptor record
	#endif


Boolean GetStringTag(const Str255 name, Str255 value);
	// returns value of string tag of the current program

void SetStringTag(const Str255 name, const Str255 value);
	// sets a string tag of the current program

Boolean GetDoubleTag(const Str255 name, double* const value);
	// returns value of double-valued tag

void SetDoubleTag(const Str255 name, double value);
	// sets a string tag of the current program



#ifndef CHECK_ERR
#define CHECK_ERR(e) if ((err=(e))!=noErr) goto done;
#endif

#ifdef __cplusplus
}
#endif

#endif
